# -*- coding: utf-8 -*-
import os,re
from ladon.exceptions.dispatcher import NonExistingAttachment,InvalidAttachmentReference

class attachment(object):
	
	def __init__(self,bufferobj,bytesize=None,headers={},header_encoding='UTF-8'):
		"""
		attachment objects consist of a binary file-like buffer object and a header
		dict. The 
		"""
		from ladon.compat import PORTABLE_BYTES
		self.headers = {}
		self.size = bytesize
		for k,v in headers.items():
			if type(k)==PORTABLE_BYTES:
				k = k.decode(header_encoding)
			if type(v)==PORTABLE_BYTES:
				v = v.decode(header_encoding)
			self.headers[k] = v
		self.bufferobj = bufferobj
		self.read = self.bufferobj.read
		if hasattr(self.bufferobj,'readline'):
			self.readline = self.bufferobj.readline
		if hasattr(self.bufferobj,'readlines'):
			self.readlines = self.bufferobj.readlines
		if hasattr(self.bufferobj,'xreadlines'):
			self.xreadlines = self.bufferobj.xreadlines
		if hasattr(self.bufferobj,'seek'):
			self.seek = self.bufferobj.seek
		if hasattr(self.bufferobj,'readinto'):
			self.readinto = self.bufferobj.readinto
		if hasattr(self.bufferobj,'flush'):
			self.flush = self.bufferobj.flush
		if hasattr(self.bufferobj,'closed'):
			self.closed = self.bufferobj.closed
		self.close = self.bufferobj.close

	def __del__(self):
		if self.bufferobj:
			self.bufferobj.close()

	def header(self,name):
		if name in self.headers:
			return self.headers[name]
		else:
			return None
	
	def headers(self):
		return self.headers


def extract_attachment_reference(a_reference,export_dict,req_encoding,ifname='',sname=''):
	rx_cid = re.compile('^cid:(.+)$',re.I)
	rx_cidx = re.compile('^cidx:(\d+)$',re.I)

	m = None
	for rx in [rx_cid,rx_cidx]:
		m = rx.match(a_reference)
		if m and rx==rx_cid:
			cid = m.groups()[0].encode(req_encoding)
			if cid and 'attachments_by_id' in export_dict and cid in export_dict['attachments_by_id']:
				attachment_info = export_dict['attachments_by_id'][cid]
				return attachment(open(attachment_info['path'],'rb'),attachment_info['size'],attachment_info['headers'],req_encoding)
			else:
				raise NonExistingAttachment(ifname,sname,'Attachment reference %s is not part of the request' % cid)
			break
		if m and rx==rx_cidx:
			cidx = int(m.groups()[0].encode(req_encoding))
			if 'attachments' in export_dict and cidx in export_dict['attachments']:
				attachment_info = export_dict['attachments'][cidx]
				return attachment(open(attachment_info['path'],'rb'),attachment_info['size'],attachment_info['headers'],req_encoding)
			else:
				raise NonExistingAttachment(ifname,sname,'Attachment with index:%d is not part of the request' % cidx)
			break
	if not m:
		raise InvalidAttachmentReference(ifname,sname,'Attachment reference %s has invalid format, must be cid:<ref> or cidx:<index>' % a_reference)




# -*- coding: utf-8 -*-

class CustomResponse(object):
	"""
	In some cases it can be nessecary to override the normal response system and return
	something completely different - ie. 
	1. if a certain method should return a file as a http attachment response for
	   browsers (Content-Disposition: attachment;).
	2. or a commandline tool that sends a SOAP request and should output raw text as
	   result
	Objects of CustomResponse descendents are intercepted in the wsgi_application part
	so the service developer has full control over response headers and data.
	"""

	def response_headers(self):
		"""
		response_headers must return a list of tuple-pairs that will constitute the response
		headers. An example::
			return [
				('Content-Disposition', 'attachment; filename="%s"' % self.filename),
				('Content-Type', 'application/force-download'),
				('Content-Length', str(self.filesize))
			]
		"""
		raise NotImplementedError("CustomResponse.response_headers must be specialized")
	
	def response_data(self):
		"""
		response_data is the data part of the response, and must be raw data (str in Python 2,
		bytes in Python 3). Iterator objects are also accepted and should be used when responses
		consist of very large data parts. For instance if the response data is a large file it
		should be returned as a iterator that is generated by reading chunks from a file object::
			self.output = open('/path/to/very/large/file.zip','rb')
			block_size = 4096
			return iter(lambda: output.read(block_size), '')

		"""
		raise NotImplementedError("CustomResponse.response_data must be specialized")



# -*- coding: utf-8 -*-
from wsgiref.util import request_uri
import os
import re
import sys
import tempfile
import traceback
import inspect
import types

from ladon.ladonizer.collection import global_service_collection
from ladon.server.dispatcher import Dispatcher
from ladon.server.customresponse import CustomResponse
from ladon.server.default.css import catalog_default_css,service_default_css
from ladon.server.default.templates import catalog_default_template,service_default_template
from ladon.tools.multiparthandler import MultiPartReader, MultiPartWriter
from ladon.interfaces import _interfaces,name_to_interface
from ladon.exceptions.dispatcher import UndefinedInterfaceName,UndefinedService
from ladon.compat import type_to_jsontype,PORTABLE_STRING_TYPES,PORTABLE_STRING
from jinja2 import Template
from docutils.core import publish_parts
from ladon.server import *
import ladon.types.tasktype as tt

if sys.version_info[0]==2:
	from StringIO import StringIO
	from urlparse import parse_qs, urlparse
	def copyFunction(f,fname):
		return types.FunctionType(f.func_code, f.func_globals, name=fname, argdefs=f.func_defaults, closure=f.func_closure)
elif sys.version_info[0]>=3:
	from urllib.parse import parse_qs, urlparse
	from io import StringIO
	def copyFunction(f,fname):
		return types.FunctionType(f.__code__,f.__globals__, name=fname, argdefs=f.__defaults__, closure=f.__closure__)

rx_ctype_charset = re.compile('charset\s*=\s*([-_.a-zA-Z0-9]+)',re.I)
rx_detect_multipart = re.compile('multipart/([^; ]+)',re.I)
rx_detect_boundary = re.compile('boundary=([^; ]+)',re.I)
rx_detect_publisher = re.compile('^@publisher: (\w+).*$',re.I)


def probe_charset(env,default='UTF-8'):
	try:
		global rx_ctype_charset
		res = rx_ctype_charset.findall(env['CONTENT_TYPE'])
		if len(res):
			return res[0]
		return env['HTTP_ACCEPT_CHARSET'].split(';')[0].split(',')[0]
	except:
		return default

def probe_client_path(environ):
	# Simplification of probe_client_path
	# Contributed by: George Marshall
	if 'HTTP_LADON_PROXY_PATH' in environ:
		return environ['HTTP_LADON_PROXY_PATH']
	return request_uri(environ,include_query=0)

def publish_doc(doc_lines):
	if len(doc_lines):
		first_line = doc_lines[0].strip()
		m = rx_detect_publisher.match(first_line)
		publisher = 'raw'
		if m:
			publisher = m.groups()[0]
			doclines = doc_lines[1:]
		else:
			doclines = doc_lines

		if publisher=='docutils':
			return publish_parts('\n'.join(doclines),writer_name='html')['body']
		if publisher=='pre':
			return '<br/>'.join(doclines)
		if publisher=='raw':
			return '\n'.join(doclines)

def read_data_file(start_response,path_info,path_list):
	ext = os.path.splitext(path_info)[1][1:]
	ct = content_types.get(ext.lower(),{}).get('Content-Type', None)
	data = b''
	status = "200 OK"
	if not ct:
		ct = "plain/text"
		status = b"501 Server Error"
		data = b'Unsupported filetype'
	else:
		found = False
		paths = map(lambda x: os.path.join(x,path_info[1:]), path_list)
		for p in paths:
			if os.path.exists(p):
				found = True
				data = open(p,'rb').read()
				break
		if not found:
			ct = "plain/text"
			data = b"File: %s could not be found" % path_info
			status = b"404 Not found"
	response_headers = [
		('Content-Type', ct ),
		('Content-Length', str(len(data)) )
	]
	start_response(status, response_headers)
	return data


class LadonWSGIApplication(object):

	reserved_roots = ['skins']

	def __init__(self,service_list,path_list=None,catalog_name=None,catalog_desc=None,logging=NO_LOGGING):
		self.staticfiles = {
			'catalog.css': {
				'data': catalog_default_css,
				'mtime': 0
			},
			'service.css': {
				'data': service_default_css,
				'mtime': 0
			},
			'catalog.template': {
				'data': catalog_default_template,
				'mtime': 0
			},
			'service.template': {
				'data': service_default_template,
				'mtime': 0
			},
			'skins/catalog-extra.css': {
				'data': '',
				'mtime': 0
			},
			'skins/service-extra.css': {
				'data': '',
				'mtime': 0
			}
		}
		self.skins = {}
		self.logging = logging
		self.catalog_name = catalog_name
		self.catalog_desc = catalog_desc
		if not catalog_name:
			self.catalog_name = "Ladon Service Catalog"
		if not catalog_desc:
			self.catalog_desc = "This is the Ladon Service Catalog. It presents the services exposed by on this particular site. Click on a service name to examine which methods and interfaces it exposes."
		if type(service_list) in PORTABLE_STRING_TYPES:
			self.service_list = [service_list]
		else:
			self.service_list = service_list

		self.path_list = path_list
		if path_list and type(path_list) in PORTABLE_STRING_TYPES:
			self.path_list = [path_list]

		if self.path_list and type(self.path_list) in [list,tuple]:
			for p in self.path_list:
				if p not in sys.path:
					sys.path += [p]

		self.import_services(self.service_list)
		self.find_custom_service_skins()


	def find_custom_service_skins(self):
		skins = {}
		for p in self.path_list:
			skins_path = os.path.join(p,'skins')
			if os.path.exists(skins_path):
				files = os.listdir(skins_path)
				for f in files:
					skin_path = os.path.join(skins_path,f)
					template_path = os.path.join(skin_path,'service.template')
					css_path = os.path.join(skin_path,'service.css')
					if os.path.isdir(skin_path) and os.path.exists(template_path) and os.path.exists(css_path) and f not in skins:
						skins[f] = skin_path
		self.skins = skins
		return skins


	def update_static(self,staticfile):
		for p in self.path_list:
			staticpath = os.path.join(p,staticfile)
			if os.path.exists(staticpath):
				static_mtime = os.stat(staticpath).st_mtime
				if staticfile not in self.staticfiles or static_mtime > self.staticfiles[staticfile]['mtime']:
					self.staticfiles[staticfile] = {}
					self.staticfiles[staticfile]['mtime'] = static_mtime
					self.staticfiles[staticfile]['data'] = open(staticpath).read()
				return True
			else:
				continue
		return False


	def generate_catalog_html(self,services,client_path,catalog_name,catalog_desc,charset):
		self.update_static('catalog.css')
		self.update_static('catalog.template')
		self.update_static('skins/catalog-extra.css')

		fix_path = urlparse(client_path)
		pretty_client_path = '%s://%s%s' % (fix_path.scheme,fix_path.netloc,fix_path.path)
		if pretty_client_path[-1:]=='/':
			pretty_client_path = pretty_client_path[:-1]
		catalog_info = {
			'catalog_name': catalog_name,
			'catalog_desc': catalog_desc,
			'css': self.staticfiles['catalog.css']['data'],
			'extra_css': self.staticfiles['skins/catalog-extra.css']['data'],
			'client_path': pretty_client_path,
			'query_string': fix_path.query,
			'charset': charset,
			'services': services.values()
		}

		template = Template(self.staticfiles['catalog.template']['data'])
		return template.render(catalog_info).encode(charset)


	def generate_service_html(self,service,client_path,charset,skin=None):
		def get_ladontype(typ):
			if type(typ)==list:
				if typ[0] in service.typemanager.type_dict:
					return typ[0].__name__
				else:
					return False
			else:
				if typ in service.typemanager.type_dict:
					return typ.__name__
				else:
					return False

		def type_to_string(typ):
			paramtype = typ
			if type(paramtype)==list:
				paramtype = paramtype[0]
				if paramtype in service.typemanager.type_dict:
					paramtype_str = '[ %s ]' % paramtype.__name__
				else:
					paramtype_str = '[ %s ]' % type_to_jsontype[paramtype]
			else:
				if paramtype in service.typemanager.type_dict:
					paramtype_str = paramtype.__name__
				elif paramtype in type_to_jsontype:
					paramtype_str = type_to_jsontype[paramtype]
				else:
					paramtype_str = paramtype.__name__
			return paramtype_str

		css_path = 'service.css'
		template_path = 'service.template'
		fix_path = urlparse(client_path)
		self.update_static('skins/service-extra.css')
		if skin:
			css_path = 'skins/%s/service.css' % skin
			template_path = 'skins/%s/service.template' % skin
			if not self.update_static(template_path) or not self.update_static(css_path):
				css_path = 'service.css'
				template_path = 'service.template'
				self.update_static(template_path)
				self.update_static(css_path)
		else:
			self.update_static(template_path)
			self.update_static(css_path)

		service_info = {
			'servicename': service.servicename,
			'doc_lines': service.doc_lines,
			'doc': publish_doc(service.doc_lines),
			'interfaces': _interfaces.keys(),
			'methods': [],
			'types': [],
			'css': self.staticfiles[css_path]['data'],
			'extra_css': self.staticfiles['skins/service-extra.css']['data'],
			'client_path': client_path,
			'query_string': fix_path.query,
			'charset': charset,
			'skins': list(self.skins.keys()),
			'current_skin': skin
		}
		for method in service.method_list():
			method_info = {
				'methodname': method.name(),
				'params': [],
				'doc': publish_doc(method._method_doc),
				'returns': {
					'type': type_to_string(method._rtype),
					'ladontype': get_ladontype(method._rtype),
					'doc': publish_doc(method._rtype_doc),
					'doc_lines': method._rtype_doc } }
			for param in method.args():
				param_info = {
					'name': param['name'],
					'type': type_to_string(param['type']),
					'ladontype': get_ladontype(param['type']),
					'optional': param['optional'],
					'doc': publish_doc(param['doc']),
					'doc_lines': param['doc'] }
				if 'default' in param:
					default_type = param['default']
					if param['type'] in PORTABLE_STRING_TYPES:
						param_info['default'] = '"%s"' % param['default']
					else:
						param_info['default'] = str(param['default'])
				method_info['params'] += [ param_info ]
			service_info['methods'] += [method_info]

		types = service_info['types']
		type_order = service.typemanager.type_order
		for typ in type_order:
			if type(typ)==dict:
				desc_type = {}
				desc_type['name'] = typ['name']
				desc_type['attributes'] = {}
				for k,v,props in typ['attributes']:
					desc_type_val = type_to_string(v)
					desc_type['attributes'][k] = {
						'type': desc_type_val,
						'props': props,
						'ladontype': get_ladontype(v) }
				types += [desc_type]

		template = Template(self.staticfiles[template_path]['data'])
		return template.render(service_info).encode(charset)

	def import_services(self,service_list):
		# Fix that eliminates the need for exec()
		# contributed by: Tamás Gulácsi
		for service in service_list:
			m = __import__(service)
			sfile = inspect.getsourcefile(m)
			for skey, smi in global_service_collection().services.items():
				for method_name,lmi in list(smi.methods.items()):
					task_rtype = getattr(lmi,'_task_rtype',None)
					if task_rtype!=None:
						if skey[0]==sfile:
							cls = getattr(m,skey[1])
							setattr(cls,"%s_progress" % method_name, tt.taskProgress)
							lmi.sinfo.add_method(tt.taskProgress,*(int,),**dict(list(tt.taskProgress_kw.items()) + [('override_method_name',"%s_progress" % method_name)]))
							setattr(cls,"%s_result" % method_name, tt.taskResult)
							lmi.sinfo.add_method(tt.taskResult,*(int,),**{'override_method_name': "%s_result" % method_name,'rtype':task_rtype})
							setattr(cls,"_task_%s" % method_name, copyFunction(getattr(cls,method_name),"_task_%s" % method_name))
							setattr(cls,method_name, copyFunction(tt.taskStarter,method_name))
	
	def parse_environ(self,environ):
		global rx_detect_multipart,rx_detect_boundary
		path_parts = []
		path_info=['']
		if 'PATH_INFO' in environ:
			path_info = environ['PATH_INFO'].strip().split('/')
		if path_info[0]=='':
			path_info = path_info[1:]
		for p in path_info:
			if p.strip():
				path_parts += [p]

		# path based schema
		sname = ifname = action = None
		if len(path_parts)>0:
			sname = path_parts[0]
		if len(path_parts)>1:
			ifname = path_parts[1]
		if len(path_parts)>2:
			action = path_parts[2]

		# Multipart detection
		multipart = boundary = None
		if 'CONTENT_TYPE' in environ:
			content_type = environ['CONTENT_TYPE']
			content_type = content_type.replace('\n','')
			multipart_match = rx_detect_multipart.findall(content_type)
			if len(multipart_match):
				multipart = multipart_match[0]
				boundary_match = rx_detect_boundary.findall(content_type)
				if len(boundary_match):
					boundary = boundary_match[0]
		return sname,ifname,action,multipart,boundary

	def __call__(self,environ, start_response):
		status = '200 OK'
		response_headers = []
		content_type = 'text/plain'
		output = ''
		charset = probe_charset(environ,default='UTF-8')

		try:
			sname,ifname,action,multipart,boundary = self.parse_environ(environ)
			client_path = probe_client_path(environ)
			query = parse_qs(environ['QUERY_STRING'])
			if sname in self.reserved_roots:
				if sname=='skins':
					data = read_data_file(start_response,environ['PATH_INFO'],self.path_list)
					return [ data ]

			sinst = ifclass = None
			if ifname:
				ifclass = name_to_interface(ifname)
				if not ifclass:
					raise UndefinedInterfaceName(ifname,'The interface name "%s" has not been defined' % ifname)

			if sname:
				service_search = global_service_collection().services_by_name(sname)
				if not len(service_search):
					raise UndefinedService(ifname,'Service "%s" has not been exposed' % sname)
				sinst = service_search[0]

			dispatcher = None
			if sinst and ifclass:
				dispatcher = Dispatcher(sinst,ifclass,charset,self.logging)
			elif not sname:
				content_type = 'text/html'
				output = self.generate_catalog_html(
					global_service_collection().services,
					client_path,
					self.catalog_name,
					self.catalog_desc,charset)
			elif sinst and not ifname:
				content_type = 'text/html'
				skin = query.get('skin', [None])[0]
				output = self.generate_service_html(sinst,client_path,charset,skin)

			if dispatcher and dispatcher.iface:
				if action=='description':
					content_type = dispatcher.iface.description_content_type()
					service_url = client_path[0:client_path.find('/description')]
					output += dispatcher.iface.description(service_url,charset,**dict(map(lambda x: (x[0],x[1][0]), query.items())))
				else:
					allowed_methods = ['POST']
					if environ['REQUEST_METHOD'] not in allowed_methods or not environ.get('CONTENT_LENGTH', ''):
						message = 'Requests for %s %s interface must be posted' % (sname,ifname)
						status = '405 %s' % message
						response_headers.append(('Allow', ','.join(allowed_methods)))
						output += message
					else:
						content_type = dispatcher.iface.response_content_type()
						content_length = int(environ['CONTENT_LENGTH'])
						if multipart and boundary:
							mph = MultiPartReader(20000,boundary.encode(charset),environ['wsgi.input'],content_length)
							mph.read_chunk()
							while not mph.eos:
								mph.read_chunk()
							encapsulated_charset = probe_charset(mph.interface_request_headers,default=None)
							request_data = mph.interface_request
							if encapsulated_charset:
								# If a specific charset is/usr/local/bin/rdesktop specified for the interface request multipart
								# let this charset superseed the charset globally specified for the request.
								dispatcher.response_encoding = encapsulated_charset

							environ['attachments'] = mph.attachments
							environ['attachments_by_id'] = mph.attachments_by_id
						else:
							request_data = environ['wsgi.input'].read(content_length)
						response_part = dispatcher.dispatch_request(request_data,environ)
						if isinstance(response_part,CustomResponse):
							response_headers += response_part.response_headers()
							start_response(status, response_headers)
							return response_part.response_data()
						elif len(environ['response_attachments'].attachments_by_cid):
							# Attachments present - Send multipart response
							response_temp_fname = tempfile.mktemp()
							temp_buffer = open(response_temp_fname,'wb')
							mpw = MultiPartWriter(temp_buffer)
							mpw.add_attachment(response_part,'%s, charset=%s' % (content_type,charset),'rpc-part')
							for cid,a in environ['response_attachments'].attachments_by_cid.items():
								mpw.add_attachment(a,'application/octet-stram',cid,a.headers)
							mpw.done()
							temp_buffer.close()
							content_length = str(os.stat(response_temp_fname).st_size)
							output = open(response_temp_fname,'rb')
							if sys.version_info[0]==2:
								content_type = "multipart/related; boundary=" + mpw.boundary
							elif sys.version_info[0]>=3:
								content_type = "multipart/related; boundary=" + str(mpw.boundary,'iso-8859-1')

						else:
							# No attachments - Send normal response
							output = response_part

		except Exception as e:
			status = '500 An Error occured while processing the request'
			content_type = 'text/plain'
			strio = StringIO()
			traceback.print_exc(file=strio)
			output = strio.getvalue()

		if 'attachments_by_id' in environ:
			for a_id,a_info in environ['attachments_by_id'].items():
				os.unlink(a_info['path'])

		if not hasattr(output,'read'):
			# not file-like object
			content_length = str(len(output))

		response_headers += [
			('Content-Type', "%s; charset=%s" % (content_type,charset)),
			('Content-Length', content_length)
		]
		start_response(status, response_headers)

		if hasattr(output,'read'):
			# File-like object
			block_size = 4096
			if 'wsgi.file_wrapper' in environ:
				return environ['wsgi.file_wrapper'](output, block_size)
			else:
				return iter(lambda: output.read(block_size), '')

		if sys.version_info[0]>=3:
			# Python 3 support
			if type(output)==str:
				output = bytes(output,charset)

		return [output]




from ladon.exceptions.base import LadonException

class LadonTypeException(LadonException):
	def __init__(self,text):
		LadonException.__init__(self)
		self.text = text
		
	def __str__(self):
		return self.text


class LadonTypePrimingException(LadonTypeException):
	def __init__(self,ladon_inst,prime_dict,text,attr_name=None):
		LadonTypeException.__init__(self,text)
		self.ladon_inst = ladon_inst
		self.prime_dict = prime_dict
		self.attr_name = attr_name
		
	def classname(self):
		return str(self.ladon_inst.__class__)
		
	def __str__(self):
		exc_info = ['classname: %s' % self.classname()]
		if self.attr_name:
			exc_info += ['attribute: %s' % self.attr_name]
		return "\n%s\n%s" % ("\n".join(exc_info),self.text)


class LadonTypeToDictException(LadonTypeException):
	def __init__(self,ladon_inst,text,attr_name=None):
		LadonTypeException.__init__(self,text)
		self.ladon_inst = ladon_inst
		self.attr_name = attr_name
		
	def classname(self):
		return str(self.ladon_inst.__class__)
		
	def __str__(self):
		exc_info = ['classname: %s' % self.classname()]
		if self.attr_name:
			exc_info += ['attribute: %s' % self.attr_name]
		return "\n%s\n%s" % ("\n".join(exc_info),self.text)


class ListAttributeMismatch(LadonTypePrimingException):
	def __init__(self,ladon_inst,prime_dict,text,attr_name=None):
		LadonTypePrimingException.__init__(self,ladon_inst,prime_dict,text,attr_name)
	

class SubitemTypeMismatch(LadonTypePrimingException):
	def __init__(self,ladon_inst,prime_dict,text,attr_name=None):
		LadonTypePrimingException.__init__(self,ladon_inst,prime_dict,text,attr_name)
	

class MandatoryAttributeMissing(LadonTypePrimingException):
	def __init__(self,ladon_inst,prime_dict,text,attr_name=None):
		LadonTypePrimingException.__init__(self,ladon_inst,prime_dict,text,attr_name)

class NonExistingAttachment(LadonTypePrimingException):
	def __init__(self,ladon_inst,prime_dict,text,attr_name=None):
		LadonTypePrimingException.__init__(self,ladon_inst,prime_dict,text,attr_name)

class LadonTypePrimerMismatch(LadonTypePrimingException):
	def __init__(self,ladon_inst,prime_dict,text,attr_name=None):
		LadonTypePrimingException.__init__(self,ladon_inst,prime_dict,text,attr_name)
	

class NeedToDefineParseTimeException(LadonTypeException):
	def __init__(self,text):
		LadonTypeException.__init__(self,text)

class UnsafeConversionDisabled(LadonTypeException):
	def __init__(self,val_type,expected_type):
		txt = "Unsafe conversions are disabled in the method called service method,\n"
		txt += "but the dispatcher encountered a potential unsafe conversion between types %s and %s.\n" % (str(val_type),str(expected_type))
		txt += 'Unsafe conversions can be enabled using the keyword argument "allow_unsafe_conversion" '
		txt += 'in the "ladonize" decorator.'
		LadonTypeException.__init__(self,txt)

class UnsafeConversionError(LadonTypeException):
	def __init__(self,val_type,expected_type,extra_txt):
		txt = "Conversion failed while doing an unsafe conversion between types %s and %s\n" % (str(val_type),str(expected_type))
		txt += extra_txt
		LadonTypeException.__init__(self,txt)

class NonUnicodeError(LadonTypeException):
	def __init__(self,val_type):
		txt = "Values must be unicode while converting from string to Ladon supported types: encountered %s" % val_type
		LadonTypeException.__init__(self,txt)


class FromUnicodeConversionError(LadonTypeException):
	def __init__(self,typ,extra_txt):
		txt = "Failed to convert from unicode string to Ladon supported type: %s\n" % str(typ)
		txt += extra_txt
		LadonTypeException.__init__(self,txt)

class ListExpected(LadonTypeToDictException):
	def __init__(self,ladon_inst,text,attr_name=None):
		LadonTypeToDictException.__init__(self,ladon_inst,text,attr_name)

class AttachmentExpected(LadonTypeToDictException):
	def __init__(self,ladon_inst,text,attr_name=None):
		LadonTypeToDictException.__init__(self,ladon_inst,text,attr_name)


class AttributeConversionException(LadonTypeToDictException):
	def __init__(self,ladon_inst,text,attr_name=None):
		LadonTypeToDictException.__init__(self,ladon_inst,text,attr_name)



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is Mozilla Universal charset detector code.
#
# The Initial Developer of the Original Code is
#          Shy Shalom
# Portions created by the Initial Developer are Copyright (C) 2005
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Mark Pilgrim - port to Python
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from .charsetprober import CharSetProber
from . import constants

# This prober doesn't actually recognize a language or a charset.
# It is a helper prober for the use of the Hebrew model probers

### General ideas of the Hebrew charset recognition ###
#
# Four main charsets exist in Hebrew:
# "ISO-8859-8" - Visual Hebrew
# "windows-1255" - Logical Hebrew 
# "ISO-8859-8-I" - Logical Hebrew
# "x-mac-hebrew" - ?? Logical Hebrew ??
#
# Both "ISO" charsets use a completely identical set of code points, whereas
# "windows-1255" and "x-mac-hebrew" are two different proper supersets of 
# these code points. windows-1255 defines additional characters in the range
# 0x80-0x9F as some misc punctuation marks as well as some Hebrew-specific 
# diacritics and additional 'Yiddish' ligature letters in the range 0xc0-0xd6.
# x-mac-hebrew defines similar additional code points but with a different 
# mapping.
#
# As far as an average Hebrew text with no diacritics is concerned, all four 
# charsets are identical with respect to code points. Meaning that for the 
# main Hebrew alphabet, all four map the same values to all 27 Hebrew letters 
# (including final letters).
#
# The dominant difference between these charsets is their directionality.
# "Visual" directionality means that the text is ordered as if the renderer is
# not aware of a BIDI rendering algorithm. The renderer sees the text and 
# draws it from left to right. The text itself when ordered naturally is read 
# backwards. A buffer of Visual Hebrew generally looks like so:
# "[last word of first line spelled backwards] [whole line ordered backwards
# and spelled backwards] [first word of first line spelled backwards] 
# [end of line] [last word of second line] ... etc' "
# adding punctuation marks, numbers and English text to visual text is
# naturally also "visual" and from left to right.
# 
# "Logical" directionality means the text is ordered "naturally" according to
# the order it is read. It is the responsibility of the renderer to display 
# the text from right to left. A BIDI algorithm is used to place general 
# punctuation marks, numbers and English text in the text.
#
# Texts in x-mac-hebrew are almost impossible to find on the Internet. From 
# what little evidence I could find, it seems that its general directionality
# is Logical.
#
# To sum up all of the above, the Hebrew probing mechanism knows about two
# charsets:
# Visual Hebrew - "ISO-8859-8" - backwards text - Words and sentences are
#    backwards while line order is natural. For charset recognition purposes
#    the line order is unimportant (In fact, for this implementation, even 
#    word order is unimportant).
# Logical Hebrew - "windows-1255" - normal, naturally ordered text.
#
# "ISO-8859-8-I" is a subset of windows-1255 and doesn't need to be 
#    specifically identified.
# "x-mac-hebrew" is also identified as windows-1255. A text in x-mac-hebrew
#    that contain special punctuation marks or diacritics is displayed with
#    some unconverted characters showing as question marks. This problem might
#    be corrected using another model prober for x-mac-hebrew. Due to the fact
#    that x-mac-hebrew texts are so rare, writing another model prober isn't 
#    worth the effort and performance hit.
#
#### The Prober ####
#
# The prober is divided between two SBCharSetProbers and a HebrewProber,
# all of which are managed, created, fed data, inquired and deleted by the
# SBCSGroupProber. The two SBCharSetProbers identify that the text is in
# fact some kind of Hebrew, Logical or Visual. The final decision about which
# one is it is made by the HebrewProber by combining final-letter scores
# with the scores of the two SBCharSetProbers to produce a final answer.
#
# The SBCSGroupProber is responsible for stripping the original text of HTML
# tags, English characters, numbers, low-ASCII punctuation characters, spaces
# and new lines. It reduces any sequence of such characters to a single space.
# The buffer fed to each prober in the SBCS group prober is pure text in
# high-ASCII.
# The two SBCharSetProbers (model probers) share the same language model:
# Win1255Model.
# The first SBCharSetProber uses the model normally as any other
# SBCharSetProber does, to recognize windows-1255, upon which this model was
# built. The second SBCharSetProber is told to make the pair-of-letter
# lookup in the language model backwards. This in practice exactly simulates
# a visual Hebrew model using the windows-1255 logical Hebrew model.
#
# The HebrewProber is not using any language model. All it does is look for
# final-letter evidence suggesting the text is either logical Hebrew or visual
# Hebrew. Disjointed from the model probers, the results of the HebrewProber
# alone are meaningless. HebrewProber always returns 0.00 as confidence
# since it never identifies a charset by itself. Instead, the pointer to the
# HebrewProber is passed to the model probers as a helper "Name Prober".
# When the Group prober receives a positive identification from any prober,
# it asks for the name of the charset identified. If the prober queried is a
# Hebrew model prober, the model prober forwards the call to the
# HebrewProber to make the final decision. In the HebrewProber, the
# decision is made according to the final-letters scores maintained and Both
# model probers scores. The answer is returned in the form of the name of the
# charset identified, either "windows-1255" or "ISO-8859-8".

# windows-1255 / ISO-8859-8 code points of interest
FINAL_KAF = '\xea'
NORMAL_KAF = '\xeb'
FINAL_MEM = '\xed'
NORMAL_MEM = '\xee'
FINAL_NUN = '\xef'
NORMAL_NUN = '\xf0'
FINAL_PE = '\xf3'
NORMAL_PE = '\xf4'
FINAL_TSADI = '\xf5'
NORMAL_TSADI = '\xf6'

# Minimum Visual vs Logical final letter score difference.
# If the difference is below this, don't rely solely on the final letter score distance.
MIN_FINAL_CHAR_DISTANCE = 5

# Minimum Visual vs Logical model score difference.
# If the difference is below this, don't rely at all on the model score distance.
MIN_MODEL_DISTANCE = 0.01

VISUAL_HEBREW_NAME = "ISO-8859-8"
LOGICAL_HEBREW_NAME = "windows-1255"

class HebrewProber(CharSetProber):
    def __init__(self):
        CharSetProber.__init__(self)
        self._mLogicalProber = None
        self._mVisualProber = None
        self.reset()

    def reset(self):
        self._mFinalCharLogicalScore = 0
        self._mFinalCharVisualScore = 0
        # The two last characters seen in the previous buffer,
        # mPrev and mBeforePrev are initialized to space in order to simulate a word 
        # delimiter at the beginning of the data
        self._mPrev = ' '
        self._mBeforePrev = ' '
        # These probers are owned by the group prober.
        
    def set_model_probers(self, logicalProber, visualProber):
        self._mLogicalProber = logicalProber
        self._mVisualProber = visualProber

    def is_final(self, c):
        return c in [FINAL_KAF, FINAL_MEM, FINAL_NUN, FINAL_PE, FINAL_TSADI]

    def is_non_final(self, c):
        # The normal Tsadi is not a good Non-Final letter due to words like 
        # 'lechotet' (to chat) containing an apostrophe after the tsadi. This 
        # apostrophe is converted to a space in FilterWithoutEnglishLetters causing 
        # the Non-Final tsadi to appear at an end of a word even though this is not 
        # the case in the original text.
        # The letters Pe and Kaf rarely display a related behavior of not being a 
        # good Non-Final letter. Words like 'Pop', 'Winamp' and 'Mubarak' for 
        # example legally end with a Non-Final Pe or Kaf. However, the benefit of 
        # these letters as Non-Final letters outweighs the damage since these words 
        # are quite rare.
        return c in [NORMAL_KAF, NORMAL_MEM, NORMAL_NUN, NORMAL_PE]
    
    def feed(self, aBuf):
        # Final letter analysis for logical-visual decision.
        # Look for evidence that the received buffer is either logical Hebrew or 
        # visual Hebrew.
        # The following cases are checked:
        # 1) A word longer than 1 letter, ending with a final letter. This is an 
        #    indication that the text is laid out "naturally" since the final letter 
        #    really appears at the end. +1 for logical score.
        # 2) A word longer than 1 letter, ending with a Non-Final letter. In normal
        #    Hebrew, words ending with Kaf, Mem, Nun, Pe or Tsadi, should not end with
        #    the Non-Final form of that letter. Exceptions to this rule are mentioned
        #    above in isNonFinal(). This is an indication that the text is laid out
        #    backwards. +1 for visual score
        # 3) A word longer than 1 letter, starting with a final letter. Final letters 
        #    should not appear at the beginning of a word. This is an indication that 
        #    the text is laid out backwards. +1 for visual score.
        # 
        # The visual score and logical score are accumulated throughout the text and 
        # are finally checked against each other in GetCharSetName().
        # No checking for final letters in the middle of words is done since that case
        # is not an indication for either Logical or Visual text.
        # 
        # We automatically filter out all 7-bit characters (replace them with spaces)
        # so the word boundary detection works properly. [MAP]

        if self.get_state() == constants.eNotMe:
            # Both model probers say it's not them. No reason to continue.
            return constants.eNotMe

        aBuf = self.filter_high_bit_only(aBuf)
        
        for cur in aBuf:
            if cur == ' ':
                # We stand on a space - a word just ended
                if self._mBeforePrev != ' ':
                    # next-to-last char was not a space so self._mPrev is not a 1 letter word
                    if self.is_final(self._mPrev):
                        # case (1) [-2:not space][-1:final letter][cur:space]
                        self._mFinalCharLogicalScore += 1
                    elif self.is_non_final(self._mPrev):
                        # case (2) [-2:not space][-1:Non-Final letter][cur:space]
                        self._mFinalCharVisualScore += 1
            else:
                # Not standing on a space
                if (self._mBeforePrev == ' ') and (self.is_final(self._mPrev)) and (cur != ' '):
                    # case (3) [-2:space][-1:final letter][cur:not space]
                    self._mFinalCharVisualScore += 1
            self._mBeforePrev = self._mPrev
            self._mPrev = cur

        # Forever detecting, till the end or until both model probers return eNotMe (handled above)
        return constants.eDetecting

    def get_charset_name(self):
        # Make the decision: is it Logical or Visual?
        # If the final letter score distance is dominant enough, rely on it.
        finalsub = self._mFinalCharLogicalScore - self._mFinalCharVisualScore
        if finalsub >= MIN_FINAL_CHAR_DISTANCE:
            return LOGICAL_HEBREW_NAME
        if finalsub <= -MIN_FINAL_CHAR_DISTANCE:
            return VISUAL_HEBREW_NAME

        # It's not dominant enough, try to rely on the model scores instead.
        modelsub = self._mLogicalProber.get_confidence() - self._mVisualProber.get_confidence()
        if modelsub > MIN_MODEL_DISTANCE:
            return LOGICAL_HEBREW_NAME
        if modelsub < -MIN_MODEL_DISTANCE:
            return VISUAL_HEBREW_NAME

        # Still no good, back to final letter distance, maybe it'll save the day.
        if finalsub < 0.0:
            return VISUAL_HEBREW_NAME

        # (finalsub > 0 - Logical) or (don't know what to do) default to Logical.
        return LOGICAL_HEBREW_NAME

    def get_state(self):
        # Remain active as long as any of the model probers are active.
        if (self._mLogicalProber.get_state() == constants.eNotMe) and \
           (self._mVisualProber.get_state() == constants.eNotMe):
            return constants.eNotMe
        return constants.eDetecting



import sys, glob
sys.path.insert(0, '..')
from chardet.universaldetector import UniversalDetector

count = 0
u = UniversalDetector()
for f in glob.glob(sys.argv[1]):
    print(f.ljust(60), end=' ')
    u.reset()
    for line in open(f, 'rb'):
        u.feed(line)
        if u.done: break
    u.close()
    result = u.result
    if result['encoding']:
        print(result['encoding'], 'with confidence', result['confidence'])
    else:
        print('******** no result')
    count += 1
print(count, 'tests')



# -*- coding: utf-8 -*-

NO_LOGGING = 0
LOG_REQUEST_ACCESS = 1
LOG_REQUEST_DICT = 2
LOG_RESPONSE_DICT = 4
LOG_EXECUTION_TIME = 8

content_types = {
	'js': {
		'Content-Type': 'application/javascript',
		'Content-Encoding': 'gzip'
	},
	'html': {
		'Content-Type': 'text/html',
		'Content-Encoding': 'gzip'
	},
	'css': {
		'Content-Type': 'text/css',
		'Content-Encoding': 'gzip'
	},
	'svg': {
		'Content-Type': 'image/svg+xml',
		'Content-Encoding': 'gzip'
	},
	'png': {
		'Content-Type': 'image/png'
	},
	'gif': {
		'Content-Type': 'image/gif'
	},
	'jpg': {
		'Content-Type': 'image/jpeg'
	},
	'bmp': {
		'Content-Type': 'image/bmp'
	},
	'ico': {
		'Content-Type': 'image/x-icon'
	}
}



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is Mozilla Universal charset detector code.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 2001
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Mark Pilgrim - port to Python
#   Shy Shalom - original C code
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from . import constants
import sys
from .latin1prober import Latin1Prober # windows-1252
from .mbcsgroupprober import MBCSGroupProber # multi-byte character sets
from .sbcsgroupprober import SBCSGroupProber # single-byte character sets
from .escprober import EscCharSetProber # ISO-2122, etc.
import re

MINIMUM_THRESHOLD = 0.20
ePureAscii = 0
eEscAscii = 1
eHighbyte = 2

class UniversalDetector:
    def __init__(self):
        self._highBitDetector = re.compile(b'[\x80-\xFF]')
        self._escDetector = re.compile(b'(\033|~{)')
        self._mEscCharSetProber = None
        self._mCharSetProbers = []
        self.reset()

    def reset(self):
        self.result = {'encoding': None, 'confidence': 0.0}
        self.done = False
        self._mStart = True
        self._mGotData = False
        self._mInputState = ePureAscii
        self._mLastChar = b''
        if self._mEscCharSetProber:
            self._mEscCharSetProber.reset()
        for prober in self._mCharSetProbers:
            prober.reset()

    def feed(self, aBuf):
        if self.done: return

        aLen = len(aBuf)
        if not aLen: return
        
        if not self._mGotData:
            # If the data starts with BOM, we know it is UTF
            if aBuf[:3] == '\xEF\xBB\xBF':
                # EF BB BF  UTF-8 with BOM
                self.result = {'encoding': "UTF-8", 'confidence': 1.0}
            elif aBuf[:4] == '\xFF\xFE\x00\x00':
                # FF FE 00 00  UTF-32, little-endian BOM
                self.result = {'encoding': "UTF-32LE", 'confidence': 1.0}
            elif aBuf[:4] == '\x00\x00\xFE\xFF': 
                # 00 00 FE FF  UTF-32, big-endian BOM
                self.result = {'encoding': "UTF-32BE", 'confidence': 1.0}
            elif aBuf[:4] == '\xFE\xFF\x00\x00':
                # FE FF 00 00  UCS-4, unusual octet order BOM (3412)
                self.result = {'encoding': "X-ISO-10646-UCS-4-3412", 'confidence': 1.0}
            elif aBuf[:4] == '\x00\x00\xFF\xFE':
                # 00 00 FF FE  UCS-4, unusual octet order BOM (2143)
                self.result = {'encoding': "X-ISO-10646-UCS-4-2143", 'confidence': 1.0}
            elif aBuf[:2] == '\xFF\xFE':
                # FF FE  UTF-16, little endian BOM
                self.result = {'encoding': "UTF-16LE", 'confidence': 1.0}
            elif aBuf[:2] == '\xFE\xFF':
                # FE FF  UTF-16, big endian BOM
                self.result = {'encoding': "UTF-16BE", 'confidence': 1.0}

        self._mGotData = True
        if self.result['encoding'] and (self.result['confidence'] > 0.0):
            self.done = True
            return

        if self._mInputState == ePureAscii:
            if self._highBitDetector.search(aBuf):
                self._mInputState = eHighbyte
            elif (self._mInputState == ePureAscii) and self._escDetector.search(self._mLastChar + aBuf):
                self._mInputState = eEscAscii

        self._mLastChar = aBuf[-1:]

        if self._mInputState == eEscAscii:
            if not self._mEscCharSetProber:
                self._mEscCharSetProber = EscCharSetProber()
            if self._mEscCharSetProber.feed(aBuf) == constants.eFoundIt:
                self.result = {'encoding': self._mEscCharSetProber.get_charset_name(),
                               'confidence': self._mEscCharSetProber.get_confidence()}
                self.done = True
        elif self._mInputState == eHighbyte:
            if not self._mCharSetProbers:
                self._mCharSetProbers = [MBCSGroupProber(), SBCSGroupProber(), Latin1Prober()]
            for prober in self._mCharSetProbers:
                if prober.feed(aBuf) == constants.eFoundIt:
                    self.result = {'encoding': prober.get_charset_name(),
                                   'confidence': prober.get_confidence()}
                    self.done = True
                    break

    def close(self):
        if self.done: return
        if not self._mGotData:
            if constants._debug:
                sys.stderr.write('no data received!\n')
            return
        self.done = True
        
        if self._mInputState == ePureAscii:
            self.result = {'encoding': 'ascii', 'confidence': 1.0}
            return self.result

        if self._mInputState == eHighbyte:
            proberConfidence = None
            maxProberConfidence = 0.0
            maxProber = None
            for prober in self._mCharSetProbers:
                if not prober: continue
                proberConfidence = prober.get_confidence()
                if proberConfidence > maxProberConfidence:
                    maxProberConfidence = proberConfidence
                    maxProber = prober
            if maxProber and (maxProberConfidence > MINIMUM_THRESHOLD):
                self.result = {'encoding': maxProber.get_charset_name(),
                               'confidence': maxProber.get_confidence()}
                return self.result

        if constants._debug:
            sys.stderr.write('no probers hit minimum threshhold\n')
            for prober in self._mCharSetProbers[0].mProbers:
                if not prober: continue
                sys.stderr.write('%s confidence = %s\n' % \
                                 (prober.get_charset_name(), \
                                  prober.get_confidence()))



# -*- coding: utf-8 -*-

catalog_default_css = '''
	body { background-color: white; margin: 0px; padding: 0px; font-size: 12px; font-family: verdana; }
	div.catName { padding: 10px; background-color: #70803C; font-size: 20px; color: white; }
	div.catDesc { padding: 20px 20px 10px; }
	ul.catService { }
	ul.catService li { padding-bottom: 10px; }
	div.catInterfaces { padding: 0px 10px 10px 30px; }
	div.catGen { text-align: center; font-style: italic; margin-top: 20px; margin-bottom: 20px; } 
'''

service_default_css = '''
	body { margin: 0px; padding: 0px; font-family: verdana; font-size: 12pt; background-color: #FCFCFC; }

	div.service-header {
		height: 34px;
		font-weight: bold; padding: 10px; font-size: 30px; color: white; border-bottom: 1px solid black; background: #70803C;
		filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#A0B06C', endColorstr='#70803C'); background: -webkit-gradient(linear, left top, left bottom, from(#A0B06C), to(#70803C)); background: -moz-linear-gradient(top,  #A0B06C,  #70803C);
		text-shadow: #555555 2px 3px 3px;
	}
	div.service-header div.service-title {position: relative; float:left}
	div.service-header .skin-selector {padding-top: 8px; margin-bottom: 0; position:relative; float: right; font-size: 12pt;}

	div.service-overview {
		float: right; margin: 20px; width: 200px;
		-moz-border-radius: 7px 7px 7px 7px; -webkit-border-radius: 7px 7px 7px 7px; border-radius: 7px 7px 7px 7px; background-color: #F6F6F6; border: 1px solid #E6E6E6; border-collapse: separate; font-size: 11px; padding: 10px; margin-right: 20px;
		-moz-box-shadow: 2px 2px 4px #555; -webkit-box-shadow: 2px 2px 4px #555; box-shadow: 2px 2px 4px #555;
	}

	div.service-overview div.headline { font-weight: bold; font-size: 18px; }
	div.service-overview div.title { font-weight: bold; margin: 4px; font-size: 1.2em; }
	div.service-overview ul.list { padding: 0px; margin: 0px; margin-left: 15px; list-style-type: none; }
	div.service-overview ul.list li { font-size: 1.1em; }

	div.service-description { padding: 15px; padding-bottom: 0px; }
	div.service-description div.title { font-weight: bold; font-size: 1.2em; }
	div.service-description div.doc-lines { font-size: 0.8em; }
	div.service-description p.url { font-size: 0.8em; font-style: italic; }
	div.service-description p.url span.url-title { font-weight: bold; }

	div.service-interfaces { padding: 15px; padding-bottom: 0px;  }
	div.service-interfaces div.title { font-weight: bold; font-size: 1.2em; }
	div.service-interfaces ul.list { font-size: 0.8em; }
	div.service-interfaces ul.list li { padding: 4px; }

	div.service-api { padding: 15px; padding-bottom: 0px;  }
	div.service-api div.methods div.title { font-weight: bold; font-size: 1.2em; } 
	div.service-api div.methods ul.list { font-size: 0.9em; list-style-type: none; } 
	div.service-api div.methods ul.list li.entry {
		-moz-border-radius: 7px 7px 7px 7px; -webkit-border-radius: 7px 7px 7px 7px; border-radius: 7px 7px 7px 7px; background-color: #F6F6F6; border: 1px solid #E6E6E6; 
		border-collapse: separate; font-size: 0.8em; padding: 10px; margin-right: 20px;
		-moz-box-shadow: 2px 2px 5px #555; -webkit-box-shadow: 2px 2px 5px #555; box-shadow: 2px 2px 5px #555;
		margin-bottom: 16px;
	} 
	div.service-api div.methods ul.list li.entry div.declaration {
		font-size: 1.5em;
	} 
	div.service-api div.methods ul.list li.entry div.declaration span.name { color: #881a1a; } 
	div.service-api div.methods ul.list li.entry span.param-type { color: #68387f; } 
	div.service-api div.methods ul.list li.entry span.param-name {} 
	div.service-api div.methods ul.list li.entry div.doc-lines { font-size: 1.2em; color: #276d11; } 
	div.service-api div.methods ul.list li.entry ul.params div.doc-lines { margin-top:0;margin-bottom:6px; font-size: 1.0em; color: #276d11; } 

	div.service-api div.types div.title { font-weight: bold; font-size: 1.2em; } 
	div.service-api div.types ul.list { font-size: 0.9em; list-style-type: none; } 
	div.service-api div.types ul.list li.entry {
		-moz-border-radius: 7px 7px 7px 7px; -webkit-border-radius: 7px 7px 7px 7px; border-radius: 7px 7px 7px 7px; background-color: #F6F6F6; border: 1px solid #E6E6E6; 
		border-collapse: separate; font-size: 0.8em; padding: 10px; margin-right: 20px;
		-moz-box-shadow: 2px 2px 5px #555; -webkit-box-shadow: 2px 2px 5px #555; box-shadow: 2px 2px 5px #555;
		margin-bottom: 16px;
	} 
	div.service-api div.types ul.list li.entry div.definition {
		font-size: 1.5em;
	} 
	div.service-api div.types ul.list li.entry div.definition span.name { color: #881a1a; } 
	div.service-api div.types ul.list li.entry span.param-type { color: #68387f; } 
	div.service-api div.types ul.list li.entry span.param-name {} 
	div.service-api div.types ul.list li.entry div.doc-lines { margin-top:0;margin-bottom:6px; font-size: 1.0em; color: #276d11; } 

	div.service-footer { font-size: 0.8em; text-align: center; font-style: italic; padding-top: 10px; padding-bottom: 10px; }

	a:link { color: #00732F; text-decoration: none }
	a:visited { color: #00732F; text-decoration: none }
	a:hover { color: #00732F; text-decoration: underline }
'''



# -*- coding: utf-8 -*-

def validate_type(typ,val):
	if [tuple,list].count(type(typ)):
		if not [tuple,list].count(type(val)):
			return False
		for i in val:
			if type(i)!=typ[0]:
				return False
	else:
		if typ!=type(val):
			return False
	return True

def get_type_info(typ):
	"""
	Convert a type to ladon type dict. If the type is a LadonType and contained in the service being processed
	the type dict will be returned, otherwise None is returned.
	"""
	from ladon.types.typemanager import TypeManager
	if not [list,tuple].count(type(typ)) and typ in TypeManager.global_type_dict:
		return TypeManager.global_type_dict[typ]
	else:
		return None

def expand_value(value,valtype, encoding='UTF-8',allow_unsafe_conversion=False,only_strings_to_unicode=True,service_name='unknown', method_name='unknown'):
	"""
	Convert the result of a method call to it's dictionary representation.
	"""
	from ladon.types.typeconverter import TypeConverter
	from ladon.tools.multiparthandler import AttachmentHandler
	from ladon.exceptions.types import AttachmentExpected
	response_attachments = AttachmentHandler()
	tc = TypeConverter(encoding=encoding,allow_unsafe_conversion=allow_unsafe_conversion,only_strings_to_unicode=only_strings_to_unicode)
	type_info = get_type_info(valtype)
	if type_info==None:
		if [list,tuple].count(type(valtype)):
			ret = []
			type_info = get_type_info(valtype[0])
			if value == valtype:
				# Assumption list attributes are always optional
				return
			
			if type_info:
				for item in value:
					ret += [item.__dict__(tc,response_attachments)]
			elif valtype[0]==attachment:
				for item in value:
					if not type(item) == attachment:
						raise AttachmentExpected(service_name,method_name,'Attachment expected got: %s' % type(item))
					ret += [response_attachments.add_attachment(item)]
			else:
				for item in value:
					ret += [tc.to_unicode_string(item,valtype[0])]
		elif valtype==attachment:
			ret = response_attachments.add_attachment(value)
		else:
			ret = tc.to_unicode_string(value,valtype)
	else:
		ret = value.__dict__(tc,response_attachments)
	
	return ret


def result_to_dict(method_info,result, encoding='UTF-8',allow_unsafe_conversion=False,only_strings_to_unicode=True):
	"""
	Convert the result of a method call to it's dictionary representation.
	"""
	from ladon.types.typeconverter import TypeConverter
	from ladon.tools.multiparthandler import AttachmentHandler
	from ladon.exceptions.types import AttachmentExpected
	response_attachments = AttachmentHandler()
	tc = TypeConverter(encoding=encoding,allow_unsafe_conversion=allow_unsafe_conversion,only_strings_to_unicode=only_strings_to_unicode)
	if method_info==None:
		res_dict = {}
		typ = type(result)
		servicename = 'None'
	else:
		res_dict = {
			'servicename': method_info.sinfo.servicename,
			'servicenumber': method_info.sinfo.servicenumber,
			'method': method_info.name()}
		typ = method_info._rtype
		servicename = method_info.sinfo.servicename
	type_info = get_type_info(typ)
	if type_info==None:
		if [list,tuple].count(type(typ)):
			result_list = []
			res_dict['result'] = result_list
			type_info = get_type_info(typ[0])
			if result == typ:
				# Assumption list attributes are always optional
				return
			
			if type_info:
				for item in result:
					result_list += [item.__dict__(tc,response_attachments)]
			elif typ[0]==attachment:
				for item in result:
					if not type(item) == attachment:
						raise AttachmentExpected('unknown',servicename,'Attachment expected got: %s' % type(item))
					result_list += [response_attachments.add_attachment(item)]
			else:
				for item in result:
					result_list += [tc.to_unicode_string(item,typ[0])]
		elif typ==attachment:
			res_dict['result'] = response_attachments.add_attachment(result)
		else:
			res_dict['result'] = tc.to_unicode_string(result,typ)
	else:
		res_dict['result'] = result.__dict__(tc,response_attachments)
	
	return res_dict



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is mozilla.org code.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Mark Pilgrim - port to Python
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from mbcharsetprober import MultiByteCharSetProber
from codingstatemachine import CodingStateMachine
from chardistribution import SJISDistributionAnalysis
from jpcntx import SJISContextAnalysis
from mbcssm import SJISSMModel
import constants, sys
from constants import eStart, eError, eItsMe

class SJISProber(MultiByteCharSetProber):
    def __init__(self):
        MultiByteCharSetProber.__init__(self)
        self._mCodingSM = CodingStateMachine(SJISSMModel)
        self._mDistributionAnalyzer = SJISDistributionAnalysis()
        self._mContextAnalyzer = SJISContextAnalysis()
        self.reset()

    def reset(self):
        MultiByteCharSetProber.reset(self)
        self._mContextAnalyzer.reset()
        
    def get_charset_name(self):
        return "SHIFT_JIS"

    def feed(self, aBuf):
        aLen = len(aBuf)
        for i in range(0, aLen):
            codingState = self._mCodingSM.next_state(aBuf[i])
            if codingState == eError:
                if constants._debug:
                    sys.stderr.write(self.get_charset_name() + ' prober hit error at byte ' + str(i) + '\n')
                self._mState = constants.eNotMe
                break
            elif codingState == eItsMe:
                self._mState = constants.eFoundIt
                break
            elif codingState == eStart:
                charLen = self._mCodingSM.get_current_charlen()
                if i == 0:
                    self._mLastChar[1] = aBuf[0]
                    self._mContextAnalyzer.feed(self._mLastChar[2 - charLen :], charLen)
                    self._mDistributionAnalyzer.feed(self._mLastChar, charLen)
                else:
                    self._mContextAnalyzer.feed(aBuf[i + 1 - charLen : i + 3 - charLen], charLen)
                    self._mDistributionAnalyzer.feed(aBuf[i - 1 : i + 1], charLen)
                    
        self._mLastChar[0] = aBuf[aLen - 1]
        
        if self.get_state() == constants.eDetecting:
            if self._mContextAnalyzer.got_enough_data() and \
                   (self.get_confidence() > constants.SHORTCUT_THRESHOLD):
                self._mState = constants.eFoundIt

        return self.get_state()

    def get_confidence(self):
        contxtCf = self._mContextAnalyzer.get_confidence()
        distribCf = self._mDistributionAnalyzer.get_confidence()
        return max(contxtCf, distribCf)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import sys,os

if sys.version_info[0]==2:
	other_version = 3
else:
	other_version = 2

#ladon_ctl_scriptname = 'scripts/ladon%d.%dctl' % sys.version_info[0:2]
#ladon_ctl_scriptname_windows = 'scripts/ladon%d.%dctl.py' % sys.version_info[0:2]

# Read the cross-version ladon-ctl
#if os.path.exists('scripts/ladon-ctl'):
	#fp = open('scripts/ladon-ctl')
	#ladon_ctl_data = fp.read()
	#fp.close()

	## Rename it to fit the particular version
	#ladon_ctl_data = "#!%s\n%s" % (sys.executable,ladon_ctl_data)
	#fp = open(ladon_ctl_scriptname,'w')
	#fp.write(ladon_ctl_data)
	#fp.close()
	#fp = open(ladon_ctl_scriptname_windows,'w')
	#fp.write(ladon_ctl_data)
	#fp.close()

readme = changes = ''
if os.path.exists('README.rst'):
	readme = open('README.rst').read()
if os.path.exists('CHANGES.rst'):
	changes = open('CHANGES.rst').read()

import imp
m=imp.load_source('init','src/ladon/__init__.py')
VERSION = '%(major)s.%(minor)s.%(micro)s' % m._version_info

SHORT_DESC = "Serve your python methods to several web service interfaces at once, including JSON-WSP, SOAP and JSON-RPC."
PACKAGES = find_packages('src')

try:
	PACKAGES.remove('chardet_py%d' % other_version)
except:
	pass
print(PACKAGES)

setup(
    name='ladon',
    packages=PACKAGES,
    package_dir={'':'src'},
    version=VERSION,
    description=SHORT_DESC,
    long_description='\n\n'.join([readme, changes]),
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Server',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',       
    ],
    keywords= ['ladonize', 'soap', 'json', 'shell', 'rpc', 'wsgi'],
    author='Jakob Simon-Gaarde',
    author_email='jakob@simon-gaarde.dk',
    maintainer = 'Jakob Simon-Gaarde',
    maintainer_email = 'jakob@simon-gaarde.dk',
    url='http://ladonize.org',
    install_requires=['jinja2','sphinx'],
    requires=['jinja2','sphinx'],
    provides=['ladon'],
    license='LGPL3',
    scripts=['scripts/ladon-ctl'],
    zip_safe=False
)
